@@ -17,13 +17,14 @@ from ruamel.yaml.nodes import *                               # NOQA
 from ruamel.yaml.nodes import (SequenceNode, MappingNode, ScalarNode)
 from ruamel.yaml.compat import (utf8, builtins_module, to_str, PY2, PY3,  # NOQA
                                 ordereddict, text_type, nprint, version_tnf, Hashable,
-                                MutableSequence)
+                                MutableSequence, MutableMapping)
 from ruamel.yaml.comments import *                               # NOQA
 from ruamel.yaml.comments import (CommentedMap, CommentedOrderedMap, CommentedSet,
-                                  CommentedKeySeq, CommentedSeq, TaggedScalar)
-from ruamel.yaml.scalarstring import *                           # NOQA
-from ruamel.yaml.scalarstring import (PreservedScalarString, SingleQuotedScalarString,
-                                      DoubleQuotedScalarString, ScalarString)
+                                  CommentedKeySeq, CommentedSeq, TaggedScalar,
+                                  CommentedKeyMap)
+from ruamel.yaml.scalarstring import (SingleQuotedScalarString, DoubleQuotedScalarString,
+                                      LiteralScalarString, FoldedScalarString,
+                                      ScalarString,)
 from ruamel.yaml.scalarint import ScalarInt, BinaryInt, OctalInt, HexInt, HexCapsInt
 from ruamel.yaml.scalarfloat import ScalarFloat
 from ruamel.yaml.timestamp import TimeStamp
@@ -76,9 +77,9 @@ class BaseConstructor(object):
         try:
             return self.loader._composer
         except AttributeError:
-            print('slt', type(self))
-            print('slc', self.loader._composer)
-            print(dir(self))
+            sys.stdout.write('slt {}\n'.format(type(self)))
+            sys.stdout.write('slc {}\n'.format(self.loader._composer))
+            sys.stdout.write('{}\n'.format(dir(self)))
             raise
 
     @property
@@ -433,9 +434,9 @@ class SafeConstructor(BaseConstructor):
             return sign * int(value_s[2:], 16)
         elif value_s.startswith('0o'):
             return sign * int(value_s[2:], 8)
-        elif self.resolver.processing_version != (1, 2) and value_s[0] == '0':
+        elif self.resolver.processing_version == (1, 1) and value_s[0] == '0':
             return sign * int(value_s, 8)
-        elif self.resolver.processing_version != (1, 2) and ':' in value_s:
+        elif self.resolver.processing_version == (1, 1) and ':' in value_s:
             digits = [int(part) for part in value_s.split(':')]
             digits.reverse()
             base = 1
@@ -1030,10 +1031,24 @@ class RoundTripConstructor(SafeConstructor):
             )
 
         if node.style == '|' and isinstance(node.value, text_type):
-            pss = PreservedScalarString(node.value)
+            lss = LiteralScalarString(node.value)
             if node.comment and node.comment[1]:
-                pss.comment = node.comment[1][0]  # type: ignore
-            return pss
+                lss.comment = node.comment[1][0]  # type: ignore
+            return lss
+        if node.style == '>' and isinstance(node.value, text_type):
+            fold_positions = []  # type: List[int]
+            idx = -1
+            while True:
+                idx = node.value.find('\a', idx + 1)
+                if idx < 0:
+                    break
+                fold_positions.append(idx - len(fold_positions))
+            fss = FoldedScalarString(node.value.replace('\a', ''))
+            if node.comment and node.comment[1]:
+                fss.comment = node.comment[1][0]  # type: ignore
+            if fold_positions:
+                fss.fold_pos = fold_positions  # type: ignore
+            return fss
         elif bool(self._preserve_quotes) and isinstance(node.value, text_type):
             if node.style == "'":
                 return SingleQuotedScalarString(node.value)
@@ -1174,7 +1189,7 @@ class RoundTripConstructor(SafeConstructor):
                 width -= 1
             e_width = len(exponent)
             e_sign = exponent[0] in '+-'
-            # print('sf', width, prec, m_sign, exp, e_width, e_sign)
+            # nprint('sf', width, prec, m_sign, exp, e_width, e_sign)
             return ScalarFloat(  # type: ignore
                 sign * float(value_s),
                 width=width,
@@ -1324,13 +1339,19 @@ class RoundTripConstructor(SafeConstructor):
             # lists are not hashable, but tuples are
             if not isinstance(key, Hashable):
                 if isinstance(key, MutableSequence):
-                    key_a = CommentedKeySeq(key)
+                    key_s = CommentedKeySeq(key)
                     if key_node.flow_style is True:
-                        key_a.fa.set_flow_style()
+                        key_s.fa.set_flow_style()
                     elif key_node.flow_style is False:
-                        key_a.fa.set_block_style()
-                    key = key_a
-                    # key = tuple(key)
+                        key_s.fa.set_block_style()
+                    key = key_s
+                elif isinstance(key, MutableMapping):
+                    key_m = CommentedKeyMap(key)
+                    if key_node.flow_style is True:
+                        key_m.fa.set_flow_style()
+                    elif key_node.flow_style is False:
+                        key_m.fa.set_block_style()
+                    key = key_m
             if PY2:
                 try:
                     hash(key)
@@ -1540,6 +1561,17 @@ class RoundTripConstructor(SafeConstructor):
                 data2.yaml_set_tag(node.tag)
                 yield data2
                 return
+            elif isinstance(node, SequenceNode):
+                data3 = CommentedSeq()
+                data3._yaml_set_line_col(node.start_mark.line, node.start_mark.column)
+                if node.flow_style is True:
+                    data3.fa.set_flow_style()
+                elif node.flow_style is False:
+                    data3.fa.set_block_style()
+                data3.yaml_set_tag(node.tag)
+                yield data3
+                data3.extend(self.construct_sequence(node))
+                return
         except:  # NOQA
             pass
         raise ConstructorError(
